import './paths';
import { MongoClient } from 'mongodb';
import ip from 'ip';

import Server from './framework/express/server';
import UserModel from './framework/mongo/user';
import CardModel from './framework/mongo/card';
import { Config } from './config';

export type Services = {
  userModel: UserModel;
  cardModel: CardModel;
};

const configFile = process.env.CONFIGFILE;
if (configFile === undefined) {
  console.log('env var CONFIGFILE not set');
  process.exit(1);
}

const config = new Config(configFile);

const mongo = new MongoClient(config.mongo.uri);
const db = mongo.db(config.mongo.dbName);

const services: Services = {
  userModel: new UserModel(db),
  cardModel: new CardModel(db),
};

const server = new Server(config.server, config.mongo, services);

process.on('SIGINT', async () => {
  console.log('\nClosing Api...');
  await mongo.close();
  await server.close();
  process.exit();
});

server.start().then(() => {
  console.log(
    `Running on http://127.0.0.1:${config.server.port} http://${ip.address()}:${
      config.server.port
    }`,
  );
});
