import { Collection, Db } from 'mongodb';
import { UpdateCardBody } from '@core';
import { Card } from '../../entities/card';
import { log } from '../../functions/logger';

class CardModel {
  private collection: Collection<Card>;

  constructor(db: Db) {
    this.collection = db.collection<Card>('cards');
  }

  public async create(tracker: string, cardInfo: Card): Promise<Card> {
    const checkCard = await this.collection.findOne({
      uuid: cardInfo.uuid,
    });
    if (checkCard) {
      log(tracker, 'Card Already Exists');
      throw new Error();
    }

    await this.collection.insertOne(cardInfo);

    const card = await this.read(tracker, cardInfo.uuid);
    log(tracker, 'CREATE USER', card);
    return card;
  }

  public async read(tracker: string, uuid: string): Promise<Card> {
    const cardDocument = await this.collection.findOne({ uuid });
    if (!cardDocument) {
      log(tracker, 'Card Not Found');
      throw new Error();
    }

    const card = new Card(cardDocument);
    log(tracker, 'READ CARD', card);
    return card;
  }

  public async update(
    tracker: string,
    uuid: string,
    cardInfo: UpdateCardBody,
  ): Promise<Card> {
    const checkCard = await this.collection.findOne({ uuid });
    if (!checkCard) {
      log(tracker, 'Card Does Not Exist');
      throw new Error();
    }

    await this.collection.updateOne(
      { uuid },
      {
        $set: {
          ...cardInfo,
          updatedAt: new Date(),
        },
      },
    );

    const card = await this.read(tracker, uuid);
    log(tracker, 'UPDATE CARD', card);
    return card;
  }

  public async list(tracker: string): Promise<Card[]> {
    const cardCollection = await this.collection.find().toArray();
    log(tracker, 'LIST CARDS');
    return cardCollection.map((cardDocument) => new Card(cardDocument));
  }
}

export default CardModel;
