import { CardInfo, CreateCardBody } from '@core';
import { Card } from '../entities/card';
import { Services } from '../app';

export function CreateCard(
  services: Services,
): (tracker: string, raw: CreateCardBody) => Promise<CardInfo> {
  const { cardModel } = services;

  return async (tracker, raw) => {
    const card = await cardModel.create(tracker, new Card(raw));
    return card.Info();
  };
}

export function ReadCard(
  services: Services,
): (tracker: string, uuid: string) => Promise<CardInfo> {
  const { cardModel } = services;

  return async (tracker, uuid) => {
    const card = await cardModel.read(tracker, uuid);
    return card.Info();
  };
}

export function ListCards(
  services: Services,
): (tracker: string) => Promise<CardInfo[]> {
  const { cardModel } = services;

  return async (tracker) => {
    const cards = await cardModel.list(tracker);
    return cards.map((card) => card.Info());
  };
}
