enum Methods {
  'GET' = 'GET',
  'POST' = 'POST',
  'PUT' = 'PUT',
}

export async function get<T>(route: string): Promise<T> {
  return request<T>('GET', route);
}

export async function post<T>(route: string, data?: unknown): Promise<T> {
  return request<T>('POST', route, data);
}

export async function put<T>(route: string, data: unknown): Promise<T> {
  return request<T>('PUT', route, data);
}

async function request<T>(
  method: keyof typeof Methods,
  route: string,
  data?: unknown,
): Promise<T> {
  const response = await fetch(`${process.env.APIURL}${route}`, {
    headers: {
      'content-type': 'application/json',
    },
    credentials: process.env.CORS ? 'include' : 'same-origin',
    mode: process.env.CORS ? 'cors' : 'same-origin',
    method: method,
    body: data ? JSON.stringify(data) : null,
  });

  if (!response.ok) throw new Error(await response.json());

  if (response.ok && response.status !== 204) {
    return response.json() as T;
  }

  return {} as T;
}
